@@ -14,7 +14,7 @@ use App::Sqitch::Types qw(Str Int Sqitch Plan Bool HashRef URI Maybe Target);
 use namespace::autoclean;
 use constant registry_release => '1.1';
 
-our $VERSION = '0.9992';
+our $VERSION = '0.9994';
 
 has sqitch => (
     is       => 'ro',
@@ -318,7 +318,7 @@ sub revert {
     # Do we want to support modes, where failures would re-deploy to previous
     # tag or all the way back to the starting point? This would be very much
     # like deploy() mode. I'm thinking not, as a failure on a revert is not
-    # something you generaly want to recover from by deploying back to where
+    # something you generally want to recover from by deploying back to where
     # you started. But maybe I'm wrong?
     $self->max_name_length(
         max map { length $_->format_name_with_tags } @changes
@@ -656,7 +656,22 @@ sub _params_for_key {
 
 sub change_id_for_key {
     my $self = shift;
-    return $self->change_id_for( $self->_params_for_key(shift) );
+    return $self->find_change_id( $self->_params_for_key(shift) );
+}
+
+sub find_change_id {
+    my ( $self, %p ) = @_;
+
+    # Find the change ID or return undef.
+    my $change_id = $self->change_id_for(
+        change_id => $p{change_id},
+        change    => $p{change},
+        tag       => $p{tag},
+        project   => $p{project} || $self->plan->project,
+    ) // return;
+
+    # Return relative to the offset.
+    return $self->change_id_offset_from_id($change_id, $p{offset});
 }
 
 sub change_for_key {
@@ -979,6 +994,11 @@ sub _check_registry {
     my $oldver = $self->registry_version;
     return $self if $newver == $oldver;
 
+    hurl engine => __x(
+        'No registry found in {destination}. Have you ever deployed?',
+        destination => $self->registry_destination,
+    ) if $oldver == 0 && !$self->initialized;
+
     hurl engine => __x(
         'Registry version is {old} but {new} is the latest known. Please upgrade Sqitch',
         old => $oldver,
@@ -1142,6 +1162,11 @@ sub change_offset_from_id {
     hurl "$class has not implemented change_offset_from_id()";
 }
 
+sub change_id_offset_from_id {
+    my $class = ref $_[0] || $_[0];
+    hurl "$class has not implemented change_id_offset_from_id()";
+}
+
 sub registered_projects {
     my $class = ref $_[0] || $_[0];
     hurl "$class has not implemented registered_projects()";
@@ -1399,10 +1424,13 @@ tables are created.
 Get, set, and clear engine variables. Variables are defined as key/value pairs
 to be passed to the engine client in calls to C<deploy> and C<revert>, if the
 client supports variables. For example, the
-L<PostgreSQL|App::Sqitch::Engine::pg> and L<Vertica|App::Sqitch::Engine::vertica>
-engines pass all the variables to their C<psql> and C<vsql> clients via the
-C<--set> option, while the L<Oracle engine|App::Sqitch::Engine::oracle> engine
-sets them via the SQL*Plus C<DEFINE> command.
+L<PostgreSQL|App::Sqitch::Engine::pg> and
+L<Vertica|App::Sqitch::Engine::vertica> engines pass all the variables to
+their C<psql> and C<vsql> clients via the C<--set> option, while the
+L<MySQL engine|App::Sqitch::Engine::mysql> engine sets them via the C<SET>
+command and the L<Oracle engine|App::Sqitch::Engine::oracle> engine sets them
+via the SQL*Plus C<DEFINE> command.
+
 
 =head3 C<deploy>
 
@@ -1559,7 +1587,7 @@ will be the offset number of changes before the latest change.
 
 =head3 C<change_for_key>
 
-  my $change = if $engine->change_for_key(key);
+  my $change = if $engine->change_for_key($key);
 
 Searches the deployed changes for a change corresponding to the specified key,
 which should be in a format as described in L<sqitchchanges>. Throws an
@@ -1568,7 +1596,7 @@ matches no changes.
 
 =head3 C<change_id_for_key>
 
-  my $change_id = if $engine->change_id_for_key(key);
+  my $change_id = if $engine->change_id_for_key($key);
 
 Searches the deployed changes for a change corresponding to the specified key,
 which should be in a format as described in L<sqitchchanges>, and returns the
@@ -1577,7 +1605,7 @@ Returns C<undef> if it matches no changes.
 
 =head3 C<change_for_key>
 
-  my $change = if $engine->change_for_key(key);
+  my $change = if $engine->change_for_key($key);
 
 Searches the list of deployed changes for a change corresponding to the
 specified key, which should be in a format as described in L<sqitchchanges>.
@@ -1644,6 +1672,13 @@ Search by change name or tag.
 The offset, if passed, will be applied relative to whatever change is found by
 the above algorithm.
 
+=head3 C<find_change_id>
+
+  my $change_id = $engine->find_change_id(%params);
+
+Like C<find_change()>, taking the same parameters, but returning an ID instead
+of a change.
+
 =head3 C<run_deploy>
 
   $engine->run_deploy($deploy_file);
@@ -2280,6 +2315,13 @@ where C<$offset> may be positive (later step) or negative (earlier step).
 Returns C<undef> if the change was not found or if the offset is more than the
 number of changes before or after the change, as appropriate.
 
+=head3 C<change_id_offset_from_id>
+
+  my $id = $engine->change_id_offset_from_id( $change_id, $offset );
+
+Like C<change_offset_from_id()> but returns the change ID rather than the
+change object.
+
 =head3 C<registry_version>
 
 Should return the current version of the target's registry.
