@@ -15,7 +15,7 @@ use namespace::autoclean;
 
 extends 'App::Sqitch::Engine';
 
-our $VERSION = '0.9992';
+our $VERSION = '0.9994';
 
 sub destination {
     my $self = shift;
@@ -108,6 +108,7 @@ has dbh => (
             Callbacks         => {
                 connected => sub {
                     my $dbh = shift;
+                    $dbh->do('SET client_min_messages = WARNING');
                     try {
                         $dbh->do(
                             'SET search_path = ?',
@@ -544,8 +545,8 @@ Initializes a database for Sqitch by installing the Sqitch registry schema.
 
 =head3 C<psql>
 
-Returns a list containing the the C<psql> client and options to be passed to
-it. Used internally when executing scripts.
+Returns a list containing the C<psql> client and options to be passed to it.
+Used internally when executing scripts.
 
 =head1 Author
 
